/*
 * Wilcard X100P FXO Interface Driver for Zapata Telephony interface
 *
 * Ported on FreeBSD by
 *      Alexander Timoshenko <gonzo@portaone.com>, PortaOne software
 *
 * Based on previous works, designs, and architectures conceived written 
 * by 
 *							 Mark Spencer <markster@linux-support.net>
 *							 Matthew Fredrickson <creslin@linux-support.net>
 *
 * Copyright (C) 2001, Linux Support Services, Inc.
 *
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/* $Id: wcfxovar.h 16 2004-10-26 04:42:50Z riz $ */


#define ZERO_BATT_RING 1
#define JAPAN 1
#define WC_MAX_IFACES 128
#define WCFXO_MAX_PORTS 4

#define RING_DEBOUNCE   64                        /* Ringer Debounce (in ms) */
#ifdef  JAPAN
#define BATT_DEBOUNCE   30                        /* Battery debounce (in ms) */
#define OH_DEBOUNCE      350                     /* Off/On hook debounce (in ms)
 */
#else
#define BATT_DEBOUNCE   80                        /* Battery debounce (in ms) */
#endif

#define MINPEGTIME        10 * 8                  /* 30 ms peak to peak gets us 
no more than 100 Hz */
#define PEGTIME          50 * 8           /* 50ms peak to peak gets us rings of 
10 Hz or more */
#define PEGCOUNT                5                          /* 5 cycles of peggin
g means RING */

#define FLAG_EMPTY  0
#define FLAG_WRITE  1
#define FLAG_READ   2

#define FLAG_INVERTSER    (1 << 0)
#define FLAG_USE_XTAL      (1 << 1)
#define FLAG_DOUBLE_CLOCK   (1 << 2)
#define FLAG_RESET_ON_AUX5  (1 << 3)

struct wcfxo_dma {
	uint32_t		*chunk;
	bus_dmamap_t		dma_map;
	bus_dma_segment_t	dma_seg;
	int			dma_nseg;
};

struct wcfxo_reg {
	unsigned long 		flags;
	unsigned char 		index;
	unsigned char 		reg;
	unsigned char 		value;
};

struct wcfxo_softc {
	struct device		sc_dv;
	bus_space_tag_t		sc_iot;
	bus_space_handle_t	sc_ioh;
	bus_addr_t		sc_iob;
	bus_size_t		sc_iosize;

	void			*sc_ih;
	bus_dma_tag_t		sc_dmat;
	pci_chipset_tag_t	sc_pct;
	struct wcfxo_dma	sc_read;
	struct wcfxo_dma	sc_write;

	char			*sc_variety;
	struct zt_span		sc_span;
	struct zt_chan		sc_chan;
	int			sc_usecount;
	int			sc_dead;
	int			sc_pos;
	unsigned long		sc_flags;
	int			sc_ring;
	int			sc_offhook;
	int			sc_battery;
	int			sc_wregcount;
	int			sc_readpos;
	int			sc_rreadpos;
	unsigned int		sc_pegtimer;
	int			sc_pegcount;
	int			sc_peg;
	int			sc_battdebounce;
	int			sc_nobatttimer;
	int			sc_ringdebounce;
#ifdef JAPAN
	int			sc_ohdebounce;
#endif
	int			sc_allread;
	/* How far off our registers are from what we expect */
	int			sc_regoffset;
	int			sc_alt;
	int			sc_ignoreread;
	/* Up to 6 registers can be written at a time */
	struct wcfxo_reg	sc_regs[ZT_CHUNKSIZE];
	struct wcfxo_reg	sc_oldregs[ZT_CHUNKSIZE];
	unsigned char		sc_lasttx[ZT_CHUNKSIZE];
	/* Up to 32 registers of whatever we most recently read */
	unsigned char		sc_readregs[32];
#ifdef ZERO_BATT_RING
	int			sc_onhook;
#endif
};

struct wcfxo_desc {
	char 			*name;
	unsigned long 		flags;
};

struct wcfxo_ident {
	pci_vendor_id_t		vendor;
	pci_product_id_t	product;
	pci_vendor_id_t		subvendor;
	const struct wcfxo_desc	*device_info;
};

static struct fxo_mode {
	char *name;
	int ohs;
	int act;
	int dct;
	int rz;
	int rt;
	int lim;
	int vol;
} fxo_modes[] =
{
	{ "FCC", 0, 0, 2, 0, 0, 0, 0 },	 /* US */
	{ "CTR21", 0, 0, 3, 0, 0, 3, 0 },   /* Austria, Belgium, Denmark, Finland, France, Germany, 
										   Greece, Iceland, Ireland, Italy, Luxembourg, Netherlands,
										   Norway, Portugal, Spain, Sweden, Switzerland, and UK */
	{ "FCC", 0, 0, 1, 0, 0, 2, 0 } /* Russia */
};
